@@ -694,9 +694,23 @@ iter_subscript(PyArrayIterObject *self, PyObject *ind)
         obj = ind;
     }
 
-    /* Any remaining valid input is an array or has been turned into one */
     if (!PyArray_Check(obj)) {
-        goto fail;
+        PyArrayObject *tmp_arr = (PyArrayObject *) PyArray_FROM_O(obj);
+        if (tmp_arr == NULL) {
+            goto fail;
+        }
+
+        if (PyArray_SIZE(tmp_arr) == 0) {
+            PyArray_Descr *indtype = PyArray_DescrFromType(NPY_INTP);
+            Py_SETREF(obj, PyArray_FromArray(tmp_arr, indtype, NPY_ARRAY_FORCECAST));
+            Py_DECREF(tmp_arr);
+            if (obj == NULL) {
+                goto fail;
+            }
+        }
+        else {
+            Py_SETREF(obj, (PyObject *) tmp_arr);
+        }
     }
 
     /* Check for Boolean array */
