@@ -1,19 +1,23 @@
 """Tests for chebyshev module.
 
 """
-from __future__ import division, absolute_import, print_function
+from functools import reduce
 
 import numpy as np
 import numpy.polynomial.chebyshev as cheb
 from numpy.polynomial.polynomial import polyval
 from numpy.testing import (
-    TestCase, assert_almost_equal, assert_raises,
-    assert_equal, assert_, run_module_suite)
+    assert_,
+    assert_almost_equal,
+    assert_equal,
+    assert_raises,
+)
 
 
 def trim(x):
     return cheb.chebtrim(x, tol=1e-6)
 
+
 T0 = [1]
 T1 = [0, 1]
 T2 = [-1, 0, 2]
@@ -28,24 +32,24 @@ T9 = [0, 9, 0, -120, 0, 432, 0, -576, 0, 256]
 Tlist = [T0, T1, T2, T3, T4, T5, T6, T7, T8, T9]
 
 
-class TestPrivate(TestCase):
+class TestPrivate:
 
     def test__cseries_to_zseries(self):
         for i in range(5):
-            inp = np.array([2] + [1]*i, np.double)
-            tgt = np.array([.5]*i + [2] + [.5]*i, np.double)
+            inp = np.array([2] + [1] * i, np.double)
+            tgt = np.array([.5] * i + [2] + [.5] * i, np.double)
             res = cheb._cseries_to_zseries(inp)
             assert_equal(res, tgt)
 
     def test__zseries_to_cseries(self):
         for i in range(5):
-            inp = np.array([.5]*i + [2] + [.5]*i, np.double)
-            tgt = np.array([2] + [1]*i, np.double)
+            inp = np.array([.5] * i + [2] + [.5] * i, np.double)
+            tgt = np.array([2] + [1] * i, np.double)
             res = cheb._zseries_to_cseries(inp)
             assert_equal(res, tgt)
 
 
-class TestConstants(TestCase):
+class TestConstants:
 
     def test_chebdomain(self):
         assert_equal(cheb.chebdomain, [-1, 1])
@@ -60,84 +64,93 @@ class TestConstants(TestCase):
         assert_equal(cheb.chebx, [0, 1])
 
 
-class TestArithmetic(TestCase):
+class TestArithmetic:
 
     def test_chebadd(self):
         for i in range(5):
             for j in range(5):
-                msg = "At i=%d, j=%d" % (i, j)
+                msg = f"At i={i}, j={j}"
                 tgt = np.zeros(max(i, j) + 1)
                 tgt[i] += 1
                 tgt[j] += 1
-                res = cheb.chebadd([0]*i + [1], [0]*j + [1])
+                res = cheb.chebadd([0] * i + [1], [0] * j + [1])
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
     def test_chebsub(self):
         for i in range(5):
             for j in range(5):
-                msg = "At i=%d, j=%d" % (i, j)
+                msg = f"At i={i}, j={j}"
                 tgt = np.zeros(max(i, j) + 1)
                 tgt[i] += 1
                 tgt[j] -= 1
-                res = cheb.chebsub([0]*i + [1], [0]*j + [1])
+                res = cheb.chebsub([0] * i + [1], [0] * j + [1])
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
     def test_chebmulx(self):
         assert_equal(cheb.chebmulx([0]), [0])
         assert_equal(cheb.chebmulx([1]), [0, 1])
         for i in range(1, 5):
-            ser = [0]*i + [1]
-            tgt = [0]*(i - 1) + [.5, 0, .5]
+            ser = [0] * i + [1]
+            tgt = [0] * (i - 1) + [.5, 0, .5]
             assert_equal(cheb.chebmulx(ser), tgt)
 
     def test_chebmul(self):
         for i in range(5):
             for j in range(5):
-                msg = "At i=%d, j=%d" % (i, j)
+                msg = f"At i={i}, j={j}"
                 tgt = np.zeros(i + j + 1)
                 tgt[i + j] += .5
                 tgt[abs(i - j)] += .5
-                res = cheb.chebmul([0]*i + [1], [0]*j + [1])
+                res = cheb.chebmul([0] * i + [1], [0] * j + [1])
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
     def test_chebdiv(self):
         for i in range(5):
             for j in range(5):
-                msg = "At i=%d, j=%d" % (i, j)
-                ci = [0]*i + [1]
-                cj = [0]*j + [1]
+                msg = f"At i={i}, j={j}"
+                ci = [0] * i + [1]
+                cj = [0] * j + [1]
                 tgt = cheb.chebadd(ci, cj)
                 quo, rem = cheb.chebdiv(tgt, ci)
                 res = cheb.chebadd(cheb.chebmul(quo, ci), rem)
                 assert_equal(trim(res), trim(tgt), err_msg=msg)
 
+    def test_chebpow(self):
+        for i in range(5):
+            for j in range(5):
+                msg = f"At i={i}, j={j}"
+                c = np.arange(i + 1)
+                tgt = reduce(cheb.chebmul, [c] * j, np.array([1]))
+                res = cheb.chebpow(c, j)
+                assert_equal(trim(res), trim(tgt), err_msg=msg)
 
-class TestEvaluation(TestCase):
+
+class TestEvaluation:
     # coefficients of 1 + 2*x + 3*x**2
     c1d = np.array([2.5, 2., 1.5])
     c2d = np.einsum('i,j->ij', c1d, c1d)
     c3d = np.einsum('i,j,k->ijk', c1d, c1d, c1d)
 
     # some random values in [-1, 1)
-    x = np.random.random((3, 5))*2 - 1
+    x = np.random.random((3, 5)) * 2 - 1
     y = polyval(x, [1., 2., 3.])
 
     def test_chebval(self):
-        #check empty input
+        # check empty input
         assert_equal(cheb.chebval([], [1]).size, 0)
 
-        #check normal input)
+        # check normal input)
         x = np.linspace(-1, 1)
         y = [polyval(x, c) for c in Tlist]
         for i in range(10):
-            msg = "At i=%d" % i
+            msg = f"At i={i}"
             tgt = y[i]
-            res = cheb.chebval(x, [0]*i + [1])
+            res = cheb.chebval(x, [0] * i + [1])
             assert_almost_equal(res, tgt, err_msg=msg)
 
-        #check that shape is preserved
+        # check that shape is preserved
         for i in range(3):
-            dims = [2]*i
+            dims = [2] * i
             x = np.zeros(dims)
             assert_equal(cheb.chebval(x, [1]).shape, dims)
             assert_equal(cheb.chebval(x, [1, 0]).shape, dims)
@@ -147,15 +160,15 @@ class TestEvaluation(TestCase):
         x1, x2, x3 = self.x
         y1, y2, y3 = self.y
 
-        #test exceptions
+        # test exceptions
         assert_raises(ValueError, cheb.chebval2d, x1, x2[:2], self.c2d)
 
-        #test values
-        tgt = y1*y2
+        # test values
+        tgt = y1 * y2
         res = cheb.chebval2d(x1, x2, self.c2d)
         assert_almost_equal(res, tgt)
 
-        #test shape
+        # test shape
         z = np.ones((2, 3))
         res = cheb.chebval2d(z, z, self.c2d)
         assert_(res.shape == (2, 3))
@@ -164,15 +177,15 @@ class TestEvaluation(TestCase):
         x1, x2, x3 = self.x
         y1, y2, y3 = self.y
 
-        #test exceptions
+        # test exceptions
         assert_raises(ValueError, cheb.chebval3d, x1, x2, x3[:2], self.c3d)
 
-        #test values
-        tgt = y1*y2*y3
+        # test values
+        tgt = y1 * y2 * y3
         res = cheb.chebval3d(x1, x2, x3, self.c3d)
         assert_almost_equal(res, tgt)
 
-        #test shape
+        # test shape
         z = np.ones((2, 3))
         res = cheb.chebval3d(z, z, z, self.c3d)
         assert_(res.shape == (2, 3))
@@ -181,50 +194,53 @@ class TestEvaluation(TestCase):
         x1, x2, x3 = self.x
         y1, y2, y3 = self.y
 
-        #test values
+        # test values
         tgt = np.einsum('i,j->ij', y1, y2)
         res = cheb.chebgrid2d(x1, x2, self.c2d)
         assert_almost_equal(res, tgt)
 
-        #test shape
+        # test shape
         z = np.ones((2, 3))
         res = cheb.chebgrid2d(z, z, self.c2d)
-        assert_(res.shape == (2, 3)*2)
+        assert_(res.shape == (2, 3) * 2)
 
     def test_chebgrid3d(self):
         x1, x2, x3 = self.x
         y1, y2, y3 = self.y
 
-        #test values
+        # test values
         tgt = np.einsum('i,j,k->ijk', y1, y2, y3)
         res = cheb.chebgrid3d(x1, x2, x3, self.c3d)
         assert_almost_equal(res, tgt)
 
-        #test shape
+        # test shape
         z = np.ones((2, 3))
         res = cheb.chebgrid3d(z, z, z, self.c3d)
-        assert_(res.shape == (2, 3)*3)
+        assert_(res.shape == (2, 3) * 3)
 
 
-class TestIntegral(TestCase):
+class TestIntegral:
 
     def test_chebint(self):
         # check exceptions
-        assert_raises(ValueError, cheb.chebint, [0], .5)
+        assert_raises(TypeError, cheb.chebint, [0], .5)
         assert_raises(ValueError, cheb.chebint, [0], -1)
         assert_raises(ValueError, cheb.chebint, [0], 1, [0, 0])
+        assert_raises(ValueError, cheb.chebint, [0], lbnd=[0])
+        assert_raises(ValueError, cheb.chebint, [0], scl=[0])
+        assert_raises(TypeError, cheb.chebint, [0], axis=.5)
 
         # test integration of zero polynomial
         for i in range(2, 5):
-            k = [0]*(i - 2) + [1]
+            k = [0] * (i - 2) + [1]
             res = cheb.chebint([0], m=i, k=k)
             assert_almost_equal(res, [0, 1])
 
         # check single integration with integration constant
         for i in range(5):
             scl = i + 1
-            pol = [0]*i + [1]
-            tgt = [i] + [0]*i + [1/scl]
+            pol = [0] * i + [1]
+            tgt = [i] + [0] * i + [1 / scl]
             chebpol = cheb.poly2cheb(pol)
             chebint = cheb.chebint(chebpol, m=1, k=[i])
             res = cheb.cheb2poly(chebint)
@@ -233,7 +249,7 @@ class TestIntegral(TestCase):
         # check single integration with integration constant and lbnd
         for i in range(5):
             scl = i + 1
-            pol = [0]*i + [1]
+            pol = [0] * i + [1]
             chebpol = cheb.poly2cheb(pol)
             chebint = cheb.chebint(chebpol, m=1, k=[i], lbnd=-1)
             assert_almost_equal(cheb.chebval(-1, chebint), i)
@@ -241,8 +257,8 @@ class TestIntegral(TestCase):
         # check single integration with integration constant and scaling
         for i in range(5):
             scl = i + 1
-            pol = [0]*i + [1]
-            tgt = [i] + [0]*i + [2/scl]
+            pol = [0] * i + [1]
+            tgt = [i] + [0] * i + [2 / scl]
             chebpol = cheb.poly2cheb(pol)
             chebint = cheb.chebint(chebpol, m=1, k=[i], scl=2)
             res = cheb.cheb2poly(chebint)
@@ -251,7 +267,7 @@ class TestIntegral(TestCase):
         # check multiple integrations with default k
         for i in range(5):
             for j in range(2, 5):
-                pol = [0]*i + [1]
+                pol = [0] * i + [1]
                 tgt = pol[:]
                 for k in range(j):
                     tgt = cheb.chebint(tgt, m=1)
@@ -261,7 +277,7 @@ class TestIntegral(TestCase):
         # check multiple integrations with defined k
         for i in range(5):
             for j in range(2, 5):
-                pol = [0]*i + [1]
+                pol = [0] * i + [1]
                 tgt = pol[:]
                 for k in range(j):
                     tgt = cheb.chebint(tgt, m=1, k=[k])
@@ -271,7 +287,7 @@ class TestIntegral(TestCase):
         # check multiple integrations with lbnd
         for i in range(5):
             for j in range(2, 5):
-                pol = [0]*i + [1]
+                pol = [0] * i + [1]
                 tgt = pol[:]
                 for k in range(j):
                     tgt = cheb.chebint(tgt, m=1, k=[k], lbnd=-1)
@@ -281,7 +297,7 @@ class TestIntegral(TestCase):
         # check multiple integrations with scaling
         for i in range(5):
             for j in range(2, 5):
-                pol = [0]*i + [1]
+                pol = [0] * i + [1]
                 tgt = pol[:]
                 for k in range(j):
                     tgt = cheb.chebint(tgt, m=1, k=[k], scl=2)
@@ -305,30 +321,30 @@ class TestIntegral(TestCase):
         assert_almost_equal(res, tgt)
 
 
-class TestDerivative(TestCase):
+class TestDerivative:
 
     def test_chebder(self):
         # check exceptions
-        assert_raises(ValueError, cheb.chebder, [0], .5)
+        assert_raises(TypeError, cheb.chebder, [0], .5)
         assert_raises(ValueError, cheb.chebder, [0], -1)
 
-        # check that zeroth deriviative does nothing
+        # check that zeroth derivative does nothing
         for i in range(5):
-            tgt = [0]*i + [1]
+            tgt = [0] * i + [1]
             res = cheb.chebder(tgt, m=0)
             assert_equal(trim(res), trim(tgt))
 
         # check that derivation is the inverse of integration
         for i in range(5):
             for j in range(2, 5):
-                tgt = [0]*i + [1]
+                tgt = [0] * i + [1]
                 res = cheb.chebder(cheb.chebint(tgt, m=j), m=j)
                 assert_almost_equal(trim(res), trim(tgt))
 
         # check derivation with scaling
         for i in range(5):
             for j in range(2, 5):
-                tgt = [0]*i + [1]
+                tgt = [0] * i + [1]
                 res = cheb.chebder(cheb.chebint(tgt, m=j, scl=2), m=j, scl=.5)
                 assert_almost_equal(trim(res), trim(tgt))
 
@@ -345,9 +361,9 @@ class TestDerivative(TestCase):
         assert_almost_equal(res, tgt)
 
 
-class TestVander(TestCase):
+class TestVander:
     # some random values in [-1, 1)
-    x = np.random.random((3, 5))*2 - 1
+    x = np.random.random((3, 5)) * 2 - 1
 
     def test_chebvander(self):
         # check for 1d x
@@ -355,7 +371,7 @@ class TestVander(TestCase):
         v = cheb.chebvander(x, 3)
         assert_(v.shape == (3, 4))
         for i in range(4):
-            coef = [0]*i + [1]
+            coef = [0] * i + [1]
             assert_almost_equal(v[..., i], cheb.chebval(x, coef))
 
         # check for 2d x
@@ -363,7 +379,7 @@ class TestVander(TestCase):
         v = cheb.chebvander(x, 3)
         assert_(v.shape == (3, 2, 4))
         for i in range(4):
-            coef = [0]*i + [1]
+            coef = [0] * i + [1]
             assert_almost_equal(v[..., i], cheb.chebval(x, coef))
 
     def test_chebvander2d(self):
@@ -393,11 +409,11 @@ class TestVander(TestCase):
         assert_(van.shape == (1, 5, 24))
 
 
-class TestFitting(TestCase):
+class TestFitting:
 
     def test_chebfit(self):
         def f(x):
-            return x*(x - 1)*(x - 2)
+            return x * (x - 1) * (x - 2)
 
         def f2(x):
             return x**4 + x**2 + 1
@@ -470,7 +486,32 @@ class TestFitting(TestCase):
         assert_almost_equal(coef1, coef2)
 
 
-class TestCompanion(TestCase):
+class TestInterpolate:
+
+    def f(self, x):
+        return x * (x - 1) * (x - 2)
+
+    def test_raises(self):
+        assert_raises(ValueError, cheb.chebinterpolate, self.f, -1)
+        assert_raises(TypeError, cheb.chebinterpolate, self.f, 10.)
+
+    def test_dimensions(self):
+        for deg in range(1, 5):
+            assert_(cheb.chebinterpolate(self.f, deg).shape == (deg + 1,))
+
+    def test_approximation(self):
+
+        def powx(x, p):
+            return x**p
+
+        x = np.linspace(-1, 1, 10)
+        for deg in range(10):
+            for p in range(deg + 1):
+                c = cheb.chebinterpolate(powx, deg, (p,))
+                assert_almost_equal(cheb.chebval(x, c), powx(x, p), decimal=12)
+
+
+class TestCompanion:
 
     def test_raises(self):
         assert_raises(ValueError, cheb.chebcompanion, [])
@@ -478,14 +519,14 @@ class TestCompanion(TestCase):
 
     def test_dimensions(self):
         for i in range(1, 5):
-            coef = [0]*i + [1]
+            coef = [0] * i + [1]
             assert_(cheb.chebcompanion(coef).shape == (i, i))
 
     def test_linear_root(self):
         assert_(cheb.chebcompanion([1, 2])[0, 0] == -.5)
 
 
-class TestGauss(TestCase):
+class TestGauss:
 
     def test_100(self):
         x, w = cheb.chebgauss(100)
@@ -495,7 +536,7 @@ class TestGauss(TestCase):
         # functions like Laguerre can be very confusing.
         v = cheb.chebvander(x, 99)
         vv = np.dot(v.T * w, v)
-        vd = 1/np.sqrt(vv.diagonal())
+        vd = 1 / np.sqrt(vv.diagonal())
         vv = vd[:, None] * vv * vd
         assert_almost_equal(vv, np.eye(100))
 
@@ -504,15 +545,15 @@ class TestGauss(TestCase):
         assert_almost_equal(w.sum(), tgt)
 
 
-class TestMisc(TestCase):
+class TestMisc:
 
     def test_chebfromroots(self):
         res = cheb.chebfromroots([])
         assert_almost_equal(trim(res), [1])
         for i in range(1, 5):
-            roots = np.cos(np.linspace(-np.pi, 0, 2*i + 1)[1::2])
-            tgt = [0]*i + [1]
-            res = cheb.chebfromroots(roots)*2**(i-1)
+            roots = np.cos(np.linspace(-np.pi, 0, 2 * i + 1)[1::2])
+            tgt = [0] * i + [1]
+            res = cheb.chebfromroots(roots) * 2**(i - 1)
             assert_almost_equal(trim(res), trim(tgt))
 
     def test_chebroots(self):
@@ -539,24 +580,24 @@ class TestMisc(TestCase):
 
     def test_cheb2poly(self):
         for i in range(10):
-            assert_almost_equal(cheb.cheb2poly([0]*i + [1]), Tlist[i])
+            assert_almost_equal(cheb.cheb2poly([0] * i + [1]), Tlist[i])
 
     def test_poly2cheb(self):
         for i in range(10):
-            assert_almost_equal(cheb.poly2cheb(Tlist[i]), [0]*i + [1])
+            assert_almost_equal(cheb.poly2cheb(Tlist[i]), [0] * i + [1])
 
     def test_weight(self):
         x = np.linspace(-1, 1, 11)[1:-1]
-        tgt = 1./(np.sqrt(1 + x) * np.sqrt(1 - x))
+        tgt = 1. / (np.sqrt(1 + x) * np.sqrt(1 - x))
         res = cheb.chebweight(x)
         assert_almost_equal(res, tgt)
 
     def test_chebpts1(self):
-        #test exceptions
+        # test exceptions
         assert_raises(ValueError, cheb.chebpts1, 1.5)
         assert_raises(ValueError, cheb.chebpts1, 0)
 
-        #test points
+        # test points
         tgt = [0]
         assert_almost_equal(cheb.chebpts1(1), tgt)
         tgt = [-0.70710678118654746, 0.70710678118654746]
@@ -567,11 +608,11 @@ class TestMisc(TestCase):
         assert_almost_equal(cheb.chebpts1(4), tgt)
 
     def test_chebpts2(self):
-        #test exceptions
+        # test exceptions
         assert_raises(ValueError, cheb.chebpts2, 1.5)
         assert_raises(ValueError, cheb.chebpts2, 1)
 
-        #test points
+        # test points
         tgt = [-1, 1]
         assert_almost_equal(cheb.chebpts2(2), tgt)
         tgt = [-1, 0, 1]
@@ -580,6 +621,3 @@ class TestMisc(TestCase):
         assert_almost_equal(cheb.chebpts2(4), tgt)
         tgt = [-1.0, -0.707106781187, 0, 0.707106781187, 1.0]
         assert_almost_equal(cheb.chebpts2(5), tgt)
-
-if __name__ == "__main__":
-    run_module_suite()
